<?php

namespace App\Api\V1\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Api\V1\Responses\ApiResponse;

/**
 * API Controller pour l'authentification.
 */
class AuthController
{
    private $authService;
    private $logService;
    private $langService;

    public function __construct()
    {
        $db = Database::getInstance();
        $this->authService = new AuthService($db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Connexion utilisateur
     * POST /api/v1/auth/login
     */
    public function login()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            return ApiResponse::methodNotAllowed();
        }

        $input = json_decode(file_get_contents('php://input'), true);
        
        if (!$input || !isset($input['username']) || !isset($input['password'])) {
            return ApiResponse::badRequest('Username and password are required');
        }

        $username = trim($input['username']);
        $password = $input['password'];
        $ipAddress = $_SERVER['REMOTE_ADDR'];

        if (empty($username) || empty($password)) {
            return ApiResponse::badRequest('Username and password cannot be empty');
        }

        if ($this->authService->login($username, $password, $ipAddress)) {
            // Génération d'un token JWT simple ou utilisation de la session
            $user = [
                'id' => $this->authService->getUserId(),
                'username' => $this->authService->getUsername(),
                'role' => $this->authService->getUserRole(),
                'token' => $this->generateSimpleToken($this->authService->getUserId())
            ];

            return ApiResponse::success($user, 'Login successful');
        } else {
            return ApiResponse::unauthorized('Invalid credentials');
        }
    }

    /**
     * Déconnexion utilisateur
     * POST /api/v1/auth/logout
     */
    public function logout()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            return ApiResponse::methodNotAllowed();
        }

        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $this->authService->logout($ipAddress);
        
        return ApiResponse::success(null, 'Logout successful');
    }

    /**
     * Récupérer les informations de l'utilisateur connecté
     * GET /api/v1/auth/me
     */
    public function me()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
            return ApiResponse::methodNotAllowed();
        }

        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized('Not authenticated');
        }

        $user = [
            'id' => $this->authService->getUserId(),
            'username' => $this->authService->getUsername(),
            'role' => $this->authService->getUserRole(),
        ];

        return ApiResponse::success($user);
    }

    /**
     * Génère un token simple (à remplacer par JWT en production)
     */
    private function generateSimpleToken($userId)
    {
        return base64_encode($userId . ':' . time() . ':' . SESSION_SECRET);
    }
}