<?php

namespace App\Api\V1\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Api\V1\Responses\ApiResponse;

/**
 * API Controller pour la gestion des certificats.
 */
class CertificatesController
{
    private $db;
    private $authService;
    private $logService;
    private $langService;

    public function __construct()
    {
        $this->db = Database::getInstance();
        $this->authService = new AuthService($this->db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Récupérer tous les certificats avec pagination
     * GET /api/v1/certificates
     */
    public function index()
    {
        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized();
        }

        $page = isset($_GET['page']) ? max(1, intval($_GET['page'])) : 1;
        $perPage = isset($_GET['per_page']) ? min(100, max(1, intval($_GET['per_page']))) : 50;
        $offset = ($page - 1) * $perPage;

        // Compter le total
        $totalStmt = $this->db->query("
            SELECT COUNT(*) as total 
            FROM certificates c 
            LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id
        ");
        $total = $totalStmt->fetch()['total'];

        // Récupérer les certificats avec pagination
        $stmt = $this->db->prepare("
            SELECT
                c.id, c.name, c.type, c.expiration_date, c.is_revoked, c.revoked_at, c.created_at,
                fp.name as perimeter_name
            FROM certificates c
            LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id
            ORDER BY fp.name IS NULL DESC, fp.name ASC, c.type DESC, c.expiration_date DESC
            LIMIT ? OFFSET ?
        ");
        $stmt->execute([$perPage, $offset]);
        $certificates = $stmt->fetchAll();

        $response = [
            'data' => $certificates,
            'current_page' => $page,
            'per_page' => $perPage,
            'total' => $total,
            'last_page' => ceil($total / $perPage)
        ];

        return ApiResponse::success($response);
    }

    /**
     * Créer un nouveau certificat
     * POST /api/v1/certificates
     */
    public function create()
    {
        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized();
        }

        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            return ApiResponse::methodNotAllowed();
        }

        $input = json_decode(file_get_contents('php://input'), true);
        
        if (!$input || !isset($input['subdomain_name']) || !isset($input['functional_perimeter_id'])) {
            return ApiResponse::badRequest('Subdomain name and functional perimeter ID are required');
        }

        $subdomainName = trim($input['subdomain_name']);
        $functionalPerimeterId = $input['functional_perimeter_id'];
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($subdomainName) || empty($functionalPerimeterId)) {
            return ApiResponse::badRequest('Subdomain name and functional perimeter are required');
        }

        // Vérifier que le périmètre existe
        $stmt = $this->db->prepare("SELECT name FROM functional_perimeters WHERE id = ?");
        $stmt->execute([$functionalPerimeterId]);
        $perimeter = $stmt->fetch();

        if (!$perimeter) {
            return ApiResponse::notFound('Functional perimeter not found');
        }

        $functionalPerimeterName = $perimeter['name'];

        // Extraire ROOT_DOMAIN du certificat CA racine pour SAN et OCSP
        $rootCaCertPath = ROOT_CA_PATH . '/certs/ca.cert.pem';
        if (!file_exists($rootCaCertPath)) {
            $this->logService->log('error', "Certificat CA racine non trouvé pour extraction ROOT_DOMAIN lors de la création de cert feuille.", $userId, $ipAddress);
            return ApiResponse::serverError('Root certificate not found');
        }

        $subjectCommand = "openssl x509 -noout -subject -in " . escapeshellarg($rootCaCertPath);
        $subjectLine = shell_exec($subjectCommand);
        $rootDomain = null;
        if ($subjectLine && preg_match('/CN\s*=\s*ca\.([^\/,\s]+)/', $subjectLine, $matches)) {
            $rootDomain = $matches[1];
        }

        if (empty($rootDomain)) {
            $this->logService->log('error', "Impossible d'extraire ROOT_DOMAIN du cert CA racine (pour SAN).", $userId, $ipAddress);
            return ApiResponse::serverError('Cannot extract root domain');
        }

        // Construire la valeur SAN
        $sanValue = "DNS:" . $subdomainName . "." . $functionalPerimeterName . "." . $rootDomain;
        $ocspUrl = OCSP_URL;

        // Préparer et exécuter la commande
        $scriptPath = SCRIPTS_PATH . '/create_cert.sh';
        $command = "OCSP_URL=" . escapeshellarg($ocspUrl) . " SAN=" . escapeshellarg($sanValue) . " " .
                   escapeshellcmd($scriptPath) . ' ' .
                   escapeshellarg($subdomainName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de création du certificat '$subdomainName' pour le périmètre '$functionalPerimeterName'.", $userId, $ipAddress);

        $output = shell_exec($command . ' 2>&1');
        $certBaseNameForCheck = $subdomainName . '.' . $functionalPerimeterName;

        if (strpos($output, "Certificat '" . $certBaseNameForCheck . "' créé avec succès :") !== false) {
            // Calculer la date d'expiration
            $certFileName = "{$subdomainName}.{$functionalPerimeterName}.cert.pem";
            $fullCertPath = INTERMEDIATE_CA_PATH_BASE . "/{$functionalPerimeterName}/certs/{$certFileName}";
            
            $expirationDate = (new \DateTime('+1 year'))->format('Y-m-d H:i:s');
            if (file_exists($fullCertPath)) {
                $certInfo = shell_exec("openssl x509 -in " . escapeshellarg($fullCertPath) . " -noout -enddate 2>/dev/null | cut -d= -f2");
                $expirationTimestamp = strtotime($certInfo);
                if ($expirationTimestamp) {
                    $expirationDate = date('Y-m-d H:i:s', $expirationTimestamp);
                }
            }

            // Enregistrer en base
            $stmt = $this->db->prepare("INSERT INTO certificates (name, type, functional_perimeter_id, expiration_date) VALUES (?, ?, ?, ?)");
            $stmt->execute([$certFileName, 'simple', $functionalPerimeterId, $expirationDate]);
            
            $certificateId = $this->db->lastInsertId();

            // Récupérer le certificat créé
            $stmt = $this->db->prepare("
                SELECT c.*, fp.name as perimeter_name 
                FROM certificates c 
                LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id 
                WHERE c.id = ?
            ");
            $stmt->execute([$certificateId]);
            $certificate = $stmt->fetch();

            $this->logService->log('info', "Certificat '{$certFileName}' créé et enregistré.", $userId, $ipAddress);
            
            return ApiResponse::created($certificate, 'Certificate created successfully');
        } else {
            $this->logService->log('error', "Échec création certificat '$subdomainName': $output", $userId, $ipAddress);
            return ApiResponse::serverError('Failed to create certificate', ['output' => $output]);
        }
    }

    /**
     * Révoquer un certificat
     * POST /api/v1/certificates/{id}/revoke
     */
    public function revoke($certificateId)
    {
        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized();
        }

        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            return ApiResponse::methodNotAllowed();
        }

        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($certificateId)) {
            return ApiResponse::badRequest('Certificate ID is required');
        }

        // Récupérer les informations du certificat
        $stmt = $this->db->prepare("
            SELECT c.name, c.type, c.is_revoked, fp.name as perimeter_name 
            FROM certificates c 
            LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id 
            WHERE c.id = ?
        ");
        $stmt->execute([$certificateId]);
        $cert = $stmt->fetch();

        if (!$cert) {
            return ApiResponse::notFound('Certificate not found');
        }

        if ($cert['type'] === 'root') {
            return ApiResponse::forbidden('ROOT certificates cannot be revoked through the interface');
        }

        if ($cert['is_revoked']) {
            return ApiResponse::badRequest('Certificate is already revoked');
        }

        // Logique de révocation selon le type
        if ($cert['type'] === 'intermediate') {
            return $this->revokeIntermediateCertificate($cert, $certificateId, $userId, $ipAddress);
        } else {
            return $this->revokeSimpleCertificate($cert, $certificateId, $userId, $ipAddress);
        }
    }

    /**
     * Statistiques des certificats
     * GET /api/v1/certificates/stats
     */
    public function stats()
    {
        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized();
        }

        // Total certificates
        $totalStmt = $this->db->query("SELECT COUNT(*) as total FROM certificates");
        $total = $totalStmt->fetch()['total'];

        // Active certificates
        $activeStmt = $this->db->query("SELECT COUNT(*) as active FROM certificates WHERE is_revoked = FALSE");
        $active = $activeStmt->fetch()['active'];

        // Revoked certificates
        $revokedStmt = $this->db->query("SELECT COUNT(*) as revoked FROM certificates WHERE is_revoked = TRUE");
        $revoked = $revokedStmt->fetch()['revoked'];

        // Certificates expiring in the next 30 days
        $expiringSoonStmt = $this->db->prepare("
            SELECT c.*, fp.name as perimeter_name 
            FROM certificates c 
            LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id 
            WHERE c.is_revoked = FALSE 
            AND c.expiration_date <= DATE_ADD(NOW(), INTERVAL 30 DAY)
            ORDER BY c.expiration_date ASC
        ");
        $expiringSoonStmt->execute();
        $expiringSoon = $expiringSoonStmt->fetchAll();

        $stats = [
            'total' => $total,
            'active' => $active,
            'revoked' => $revoked,
            'expiring_soon' => $expiringSoon
        ];

        return ApiResponse::success($stats);
    }

    /**
     * Télécharger un certificat
     * GET /api/v1/certificates/download
     */
    public function download()
    {
        if (!$this->authService->isLoggedIn()) {
            return ApiResponse::unauthorized();
        }

        $type = $_GET['type'] ?? null;
        $fileName = $_GET['file'] ?? null;
        $perimeterName = $_GET['perimeter'] ?? null;

        $userId = $this->authService->getUserId();
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        
        $this->logService->log('info', "Download attempt: type='{$type}', file='{$fileName}', perimeter='{$perimeterName}'", $userId, $ipAddress);

        if (empty($type) || empty($fileName)) {
            return ApiResponse::badRequest('Missing download parameters');
        }

        if (basename($fileName) !== $fileName || ($perimeterName && basename($perimeterName) !== $perimeterName)) {
            return ApiResponse::badRequest('Invalid characters in file or perimeter name');
        }

        $filePath = $this->buildFilePath($type, $fileName, $perimeterName);
        
        if (!$filePath) {
            return ApiResponse::badRequest('Invalid certificate type or file');
        }

        if (!file_exists($filePath) || !is_readable($filePath)) {
            return ApiResponse::notFound('File not found or not readable');
        }

        // Check permissions for private keys
        if (str_ends_with($fileName, '.key.pem') && $this->authService->getUserRole() !== 'admin') {
            return ApiResponse::forbidden('Unauthorized to download private keys');
        }

        $this->logService->log('info', "File '{$filePath}' download initiated", $userId, $ipAddress);

        // Set headers for file download
        $mimeType = 'application/x-pem-file';
        header('Content-Description: File Transfer');
        header('Content-Type: ' . $mimeType);
        header('Content-Disposition: attachment; filename="' . basename($filePath) . '"');
        header('Expires: 0');
        header('Cache-Control: must-revalidate');
        header('Pragma: public');
        header('Content-Length: ' . filesize($filePath));

        if (ob_get_level()) {
            ob_end_clean();
        }

        readfile($filePath);
        exit;
    }

    private function buildFilePath($type, $fileName, $perimeterName)
    {
        switch ($type) {
            case 'root':
                if ($fileName === 'ca.cert.pem') {
                    return ROOT_CA_PATH . '/certs/' . $fileName;
                } elseif ($fileName === 'ca.key.pem') {
                    return ROOT_CA_PATH . '/private/' . $fileName;
                }
                break;
            
            case 'intermediate':
            case 'simple':
                if (empty($perimeterName)) {
                    return null;
                }
                
                if (str_ends_with($fileName, '.key.pem')) {
                    return INTERMEDIATE_CA_PATH_BASE . '/' . $perimeterName . '/private/' . $fileName;
                } else {
                    return INTERMEDIATE_CA_PATH_BASE . '/' . $perimeterName . '/certs/' . $fileName;
                }
                break;
        }
        
        return null;
    }

    private function revokeIntermediateCertificate($cert, $certificateId, $userId, $ipAddress)
    {
        $functionalPerimeterName = $cert['perimeter_name'];
        $intermediateCertPath = "/opt/tls/intermediate/" . $functionalPerimeterName . "/certs/" . $cert['name'];
        $rootCaConfigPath = "/opt/tls/root/openssl.cnf";
        $rootCaCrlPath = "/opt/tls/root/crl/crl.pem";

        $revokeCmd = sprintf(
            "openssl ca -batch -config %s -revoke %s",
            escapeshellarg($rootCaConfigPath),
            escapeshellarg($intermediateCertPath)
        );

        $this->logService->log('info', "Tentative de révocation du certificat intermédiaire '{$cert['name']}'", $userId, $ipAddress);
        $outputRevoke = shell_exec($revokeCmd . ' 2>&1');

        if (strpos($outputRevoke, "Data Base Updated") !== false || strpos($outputRevoke, "Successfully revoked certificate") !== false) {
            $generateCrlCmd = sprintf(
                "openssl ca -batch -config %s -gencrl -out %s",
                escapeshellarg($rootCaConfigPath),
                escapeshellarg($rootCaCrlPath)
            );

            $outputCrl = shell_exec($generateCrlCmd . ' 2>&1');

            if ((strpos($outputCrl, "CRL Generated") !== false || strpos($outputCrl, "CRL generated") !== false) && file_exists($rootCaCrlPath)) {
                $stmt_update = $this->db->prepare("UPDATE certificates SET is_revoked = TRUE, revoked_at = NOW() WHERE id = ?");
                $stmt_update->execute([$certificateId]);

                $this->logService->log('info', "Certificat intermédiaire '{$cert['name']}' révoqué et CRL mise à jour", $userId, $ipAddress);
                return ApiResponse::success(null, 'Intermediate certificate revoked successfully');
            } else {
                $this->logService->log('error', "Échec de la mise à jour de la CRL: $outputCrl", $userId, $ipAddress);
                return ApiResponse::serverError('Failed to update CRL');
            }
        } else {
            $this->logService->log('error', "Échec révocation certificat intermédiaire: $outputRevoke", $userId, $ipAddress);
            return ApiResponse::serverError('Failed to revoke intermediate certificate');
        }
    }

    private function revokeSimpleCertificate($cert, $certificateId, $userId, $ipAddress)
    {
        $certBaseName = str_replace('.cert.pem', '.cert', $cert['name']);
        $functionalPerimeterName = $cert['perimeter_name'];

        $command = escapeshellcmd(SCRIPTS_PATH . '/revoke_cert.sh') . ' ' .
                   escapeshellarg($certBaseName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de révocation du certificat simple '{$cert['name']}'", $userId, $ipAddress);
        $output = shell_exec($command . ' 2>&1');

        if (strpos($output, "Certificat '$certBaseName' révoqué avec succès.") !== false) {
            $stmt_update = $this->db->prepare("UPDATE certificates SET is_revoked = TRUE, revoked_at = NOW() WHERE id = ?");
            $stmt_update->execute([$certificateId]);

            $this->logService->log('info', "Certificat simple '{$cert['name']}' révoqué", $userId, $ipAddress);
            return ApiResponse::success(null, 'Certificate revoked successfully');
        } else {
            $this->logService->log('error', "Échec révocation certificat simple: $output", $userId, $ipAddress);
            return ApiResponse::serverError('Failed to revoke certificate');
        }
    }
}